{-------------------------------------------------------------------------------
Delphi Save And Restore
Copyright (C) 2009-2010 ouiouioui contact@toutenvrac.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-------------------------------------------------------------------------------}
Unit Utils;

Interface

Uses
  Windows, Messages, SysUtils, Classes, TLHelp32, ZipMstr19, Dialogs, Forms, ShellApi;

Const
  // constante utilise pour savoir comment est la DDL Zip
  DLLZIP_ALREADYLOADED = 0;
  DLLZIP_ERREXTRACT    = 1;
  DLLZIP_LOADNOW       = 2;
  DLLZIP_UNLOADNOW     = 3;

Function TempDirectory: String;
Function SystemDirectory: String;
Function LoadDllZipNow(Var ZipMaster: TZipmaster19; Load: Boolean = True): Integer;
Function ExtractOfRes(ResName, FileName: String): Boolean;
Function ApplicationVersion: String;
Function ApplicationVersionBuild: String;
Function DelDir(Dir: String): Boolean;
Function WaitExec(FileName, Parameters: String; Hidden: Boolean = True; TimeOut: Integer = 0): Longword;

Implementation

Function TempDirectory: String;
Begin
  Result := IncludeTrailingPathDelimiter(GetEnvironmentVariable('TEMP'));
  If Not DirectoryExists(Result) Then
  Begin
    Result := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName));
  End;
End;

Function SystemDirectory: String;
Const
  dwLength: DWORD = 255;
Var
  SystemDir: PChar;
Begin
  GetMem(SystemDir, dwLength);
  GetSystemDirectory(SystemDir, dwLength);
  Result := IncludeTrailingPathDelimiter(String(SystemDir));
  FreeMem(SystemDir, dwLength);
End;

//{$R DelZip179.RES} //ceci permet d'inclure la DLL DelZip179.DLL

// permet de vrifier s'il y a besoin ou non de charger la DLL ZIP
Function LoadDllZipNow(Var ZipMaster: TZipmaster19; Load: Boolean = True): Integer;
Var
  DllZipMasterName: String;
Begin
  DllZipMasterName := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)) + 'DelZip179.dll'; // on rcupre le chemin temporaire et on ajoute le nom de la DLL ZIP
  If Load Then // l'utilisateur veut l'utiliser on vrifie qu'elle est charge sinon ont la charge
  Begin
    If FileExists(DllZipMasterName) And ZipMaster.DLL_Load Then // on vrifie que le fichier existe et qu'elle est dj charge
      Result := DLLZIP_ALREADYLOADED // on renvoie comme rponse quel est dj charg
    Else
    If Not FileExists(DllZipMasterName) And Not ExtractOfRes('DelZip179', DllZipMasterName) Then // si le fichier existe pas on essaye de l'extraire
      Result := DLLZIP_ERREXTRACT // on renvoie comme rponse impossible de l'extraire
    Else
    Begin
      ZipMaster.DLLDirectory := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)); // le fichier existe et elle n'est pas charge alors on effectue le chargement
      ZipMaster.DLL_Load     := True;
      Result                 := DLLZIP_LOADNOW; //on renvoie comme rponse que l'on vient de charger la DLL
    End;
  End
  // utilisateur veut la dcharger alors on vrifie qu'elle est charg
  Else
  If ZipMaster.Dll_Load Then
  Begin
    ZipMaster.Dll_Load := False;  // la DLL est charg ont la dcharge
    DeleteFile(DllZipMasterName); // puis on efface le fichier
    Result := DLLZIP_UNLOADNOW;   // on renvoie comme rponse que la dll a t dcharg
  End
  Else
    Result := -1; // erreur l'utilisateur veut la dcharger mais elle n'est pas charge on renvoie -1
End;

Function ExtractOfRes(ResName, FileName: String): Boolean;
Var
  ResourceStream: TResourceStream;
  FichierStream:  TFileStream;
Begin
  Result := False;
  If FileExists(FileName) Then // on vrifie que le fichier  extraire n'existe pas dj
    DeleteFile(FileName); // sinon on le supprime
  If Not FileExists(FileName) Then // si le fichier n'existe pas alors on l'extrait
  Begin
    ResourceStream := TResourceStream.Create(hInstance, ResName, RT_RCDATA); // cration d'un flux mmoire donnant accs  la ressource compil
    Try
      FichierStream := TFileStream.Create(FileName, fmCreate); // FileStream permet aux applications de lire et crire dans un fichier disque.
      Try
        FichierStream.CopyFrom(ResourceStream, 0); // copie du flux donnant accs aux ressources vers le flux fichier
        If FileExists(FileName) Then
          Result := True
        Else
          Result := False;
      Finally
        FichierStream.Free; //libration mmoire du stream fichier
      End;
    Finally
      ResourceStream.Free; //libration mmoire du flux mmoire
    End;
  End;
End;

Function ApplicationVersion: String;
Var
  VerInfoSize, VerValueSize, Dummy: DWord;
  VerInfo:                          Pointer;
  VerValue:                         PVSFixedFileInfo;
Begin
  VerInfoSize := GetFileVersionInfoSize(PChar(ParamStr(0)), Dummy);
  If VerInfoSize <> 0 Then // Les info de version sont inclues
  Begin
    GetMem(VerInfo, VerInfoSize); // On alloue de la mmoire pour un pointeur sur les info de version
    GetFileVersionInfo(PChar(ParamStr(0)), 0, VerInfoSize, VerInfo); // On rcupre ces informations
    VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize); // On traite les informations ainsi rcupres
    With VerValue^ Do // on formate le rsultat en sparant chaque numro d'un point
    Begin
      Result := IntToStr(dwFileVersionMS Shr 16);
      Result := Result + '.' + IntToStr(dwFileVersionMS And $FFFF);
      Result := Result + '.' + IntToStr(dwFileVersionLS Shr 16);
      Result := Result + '.' + IntToStr(dwFileVersionLS And $FFFF);
    End;
    FreeMem(VerInfo, VerInfoSize); // On libre la place prcdemment alloue
  End
  Else
    Raise EAccessViolation.Create('Les informations de version de sont pas inclues'); // Les infos de version ne sont pas inclues On dclenche une exception dans le programme
End;

Function ApplicationVersionBuild: String;
Var
  PointBuild: Integer;
  AppVer:     String;
Begin
  Result     := '';
  AppVer     := ApplicationVersion;
  PointBuild := LastDelimiter('.', AppVer);
  Result     := 'Version ' + Copy(AppVer, 0, PointBuild - 1) + ' Build : ' + Copy(AppVer, PointBuild + 1, Length(AppVer));
End;

// permet d'effacer un dossier ainsi que les sous dossiers et fichiers inclus dedans
Function DelDir(Dir: String): Boolean;
Var
  iIndex:    Integer;
  SearchRec: TSearchRec;
  sFileName: String;
Begin
  Dir    := IncludeTrailingPathDelimiter(Dir);
  iIndex := FindFirst(Dir + '*.*', faAnyFile, SearchRec);
  If iIndex = 0 Then
  Begin
    Repeat
      sFileName := Dir + SearchRec.Name;
      If DirectoryExists(sFileName){SearchRec.Attr = faDirectory} Then //si l'objet est un dossier
      Begin
        If (SearchRec.Name <> '') And
          (SearchRec.Name <> '.') And
          (SearchRec.Name <> '..') Then //si l'objet n'est pas vide et ce n'est pas les dossiers systmes pour remonte
          DelDir(sFileName); //alors on l'efface
      End
      Else
      Begin
        If SearchRec.Attr <> faArchive Then
          FileSetAttr(sFileName, faArchive);
        DeleteFile(sFileName);
      End;
      iIndex := FindNext(SearchRec); //on cherche le suivant
    Until iIndex <> 0;
    FindClose(SearchRec); //on ferme le dossier
    Result := RemoveDir(ExtractFileDir(Dir)); //on efface le dossier de base qui est maintenant vide normalement
  End
  Else
    Result := False;
End;

//permet d'excuter un fichier avec des paramtres et d'attendre qu'il ait fini
//en rcuprant son code de sortie en rsultat, possibilit de dfinir un temps en millisecondes
//aprs lequel le processus est tu et le rsultat renvoy est alors 9999
Function WaitExec(FileName, Parameters: String; Hidden: Boolean = True; TimeOut: Integer = 0): Longword;
Var
  ProcInfo:  TProcessInformation;
  StartInfo: TStartupInfo;
  ExitCode:  Longword;
  TimeNow:   Integer;
Begin
  TimeNow  := 0;
  ExitCode := 9999;
  Result   := ExitCode;
  FillChar(ProcInfo, SizeOf(TProcessInformation), 0);
  FillChar(StartInfo, SizeOf(TStartupInfo), 0);
  StartInfo.cb := SizeOf(TStartupInfo);
  If Hidden Then
  Begin
    StartInfo.dwFlags     := STARTF_USESHOWWINDOW Or STARTF_USESTDHANDLES;
    StartInfo.wShowWindow := SW_HIDE;
  End;
  Try
    If CreateProcess(NIL, PChar(FileName + ' ' + Parameters), NIL, NIL, False,
      CREATE_DEFAULT_ERROR_MODE + NORMAL_PRIORITY_CLASS, NIL, NIL, StartInfo, ProcInfo) Then
    Begin
      Repeat
        If TimeOut > 0 Then
        Begin
          If TimeNow >= TimeOut Then
          Begin
            ExitCode := 9999;
            Result   := ExitCode;
            TerminateProcess(ProcInfo.hProcess, 0);
            Exit;
          End
          Else
            Inc(TimeNow, 100);
        End;
        Sleep(100);
        Application.ProcessMessages;
        GetExitCodeProcess(ProcInfo.hProcess, ExitCode);
      Until ExitCode <> STILL_ACTIVE;
      Result := ExitCode;
    End
    Else
      Application.MessageBox(PChar('Impossible d''executer l''application ' + FileName), PChar('Erreur'), MB_ICONERROR);
  Finally
    CloseHandle(ProcInfo.hProcess);
  End;
End;

End.
